package com.ekexiu.project.bridge.envdata;

import com.ekexiu.project.bridge.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

/**
 * Created by TT on 2019/3/8.
 */
@Path("/envData")
public class EnvDataService {
    private AtomicReference<List<EnvData>> lastData = new AtomicReference<List<EnvData>>();
    public static AtomicBoolean dolog=new AtomicBoolean(false);

    @Autowrie
    private EnvDataDao envDataDao;

    public EnvDataDao getEnvDataDao() {
        return envDataDao;
    }

    public void setEnvDataDao(EnvDataDao envDataDao) {
        this.envDataDao = envDataDao;
    }

    public void setLastData(List<EnvData> lastData) {
        if (lastData.size() == 0) {
            return;
        }
        this.lastData.set(lastData);
    }

    @Path("/openlog")
    @Get
    public void openlog() {
        dolog.set(true);
    }

    @Path("/closelog")
    @Get
    public void closelog() {
        dolog.set(false);
    }

    @Path("/byTime")
    @Get
    public List<EnvData> query(@JdbcConn Connection con, @DefaultValue("\"00000000000000\"") String bt, @DefaultValue("\"99999999999999\"") String et) throws SQLException {
        return envDataDao.query(con, bt, et);
    }

    @Path
    @Get
    public List<EnvData> query(@JdbcConn Connection con) throws SQLException {
        List<EnvData> res = lastData.get();
        if (res == null) {
            res = envDataDao.query(con);
            this.lastData.set(res);
        }
        return res;
    }


    @Path("/server")
    @Post
    public String insert(@JdbcConn(true)Connection con, @LoginUser SessionUser sessionUser, EnvServer envServer)throws SQLException {
        String id = StringUtil.buildUUID();
        envServer.setId(id);
        envServer.setActive(true);
        envServer.setCreator(sessionUser.getId());
        envServer.setModifier(sessionUser.getId());
        envDataDao.insert(con, envServer);
        return id;
    }

    @Path("/transducer")
    @Post
    public String insert(@JdbcConn(true)Connection con,@LoginUser SessionUser sessionUser,EnvTransducer envTransducer)throws SQLException {
        String id = StringUtil.buildUUID();
        envTransducer.setId(id);
        envTransducer.setActive(true);
        envTransducer.setCreator(sessionUser.getId());
        envTransducer.setModifier(sessionUser.getId());
        envDataDao.insert(con, envTransducer);
        return id;
    }

    @Path("/server/delete")
    @Get
    public void deleteServer(@JdbcConn(true)Connection con,@LoginUser SessionUser sessionUser,String id)throws SQLException {
        envDataDao.logicDeleteServer(con, sessionUser.getId(), id);
        String[] serverId = new String[1];
        serverId[0] = id;
        envDataDao.logicDeleteTranByServers(con, sessionUser.getId(), serverId);
    }

    @Path("/transducer/delete")
    @Get
    public void deleteTran(@JdbcConn(true)Connection con, @LoginUser SessionUser sessionUser,String id)throws SQLException {
        envDataDao.logicDeleteTransducer(con, sessionUser.getId(), id);
    }

    @Path("/server/pq")
    @Get
    public PageQueryResult<EnvServer> pageQuery(@JdbcConn Connection con, @Nullable Boolean active,@Nullable String key,@Nullable String code,int pageSize,int pageNo)throws SQLException {
        return envDataDao.pageQuery(con, active, key == null ? null : "%" + key + "%", code == null ? null : "%" + code + "%", pageSize, pageNo);
    }

    @Path("/transducer/pq")
    @Get
    public PageQueryResult<EnvTransducer> pageQuery(@JdbcConn Connection con,@Nullable Boolean active,@Nullable String serverCode,@Nullable String bridgeName,@Nullable String system,@Nullable Integer code,int pageSize,int pageNo)throws SQLException {
        return envDataDao.pageQuery(con, active, serverCode == null ? null : "%" + serverCode + "%", bridgeName == null ? null : "%" + bridgeName + "%", system, code, pageSize, pageNo);
    }

    @Path("/server/qo")
    @Get
    public EnvServer queryServer(@JdbcConn Connection con,String id)throws SQLException {
        return envDataDao.queryEnvServer(con, id);
    }

    @Path("/transducer/qo")
    @Get
    public EnvTransducer queryTransducer(@JdbcConn Connection con,String id)throws SQLException {
        return envDataDao.queryEnvTransducer(con, id);
    }

    @Get
    @Path("/server/checkSeq")
    public EnvServer serverBySeq(@JdbcConn Connection con, String seq, @Nullable String id, @Nullable Boolean active)throws SQLException {
        return this.envDataDao.querySerBySeq(con, seq,id,active);
    }

    @Get
    @Path("/server/checkCode")
    public EnvServer serverByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.envDataDao.querySerByCode(con, code, id,active);
    }

    @Get
    @Path("/transducer/checkSeq")
    public EnvServer transducerBySeq(@JdbcConn Connection con, String seq, @Nullable String id, @Nullable Boolean active)throws SQLException {
        return this.envDataDao.queryTraBySeq(con, seq,id,active);
    }

    @Get
    @Path("/transducer/checkCode")
    public EnvServer transducerByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.envDataDao.queryTraByCode(con, code, id,active);
    }

    @Path("/all/byBridge")
    @Get
    @LoginUser
    public Map<String,Object> queryAll(@JdbcConn Connection con,@Nullable Boolean active,String bridgeId)throws SQLException {
        Map<String, Object> result = new HashMap<>();
        List<EnvServer> servers = envDataDao.queryByBridge(con, active, bridgeId);
        result.put("server", servers);
        if (servers.size() > 0) {
            String[] serverIds = new String[servers.size()];
            for (int i = 0; i < servers.size(); ++i) {
                serverIds[i] = servers.get(i).getId();
            }
            List<EnvTranLastValue> transducers = envDataDao.queryByServer(con, active, serverIds);
            for (EnvTranLastValue transducer : transducers) {
                final int id = transducer.getSeq();
                transducer.setLastValue(JdbcUtil.queryDouble(con, "select * from env_data WHERE item_id = ? ORDER BY create_time desc limit 1", new PreparedStatementConfig() {
                    @Override
                    public void config(PreparedStatement preparedStatement) throws SQLException {
                        preparedStatement.setInt(1,id);
                    }
                },0));
            }
            result.put("transducer", transducers);
        } else {
            result.put("transducer", servers);
        }
        return result;
    }




}
